use strict;
use warnings;

# this test was generated with Dist::Zilla::Plugin::Test::EOL 0.18

use Test::More 0.88;
use Test::EOL;

my @files = (
    'lib/Dist/Iller.pm',
    'lib/Dist/Iller/Builder.pm',
    'lib/Dist/Iller/Configuration.pm',
    'lib/Dist/Iller/Configuration/Plugin.pm',
    'lib/Dist/Iller/Configuration/Prereq.pm',
    'lib/Dist/Iller/Doctype.pm',
    'lib/Dist/Iller/Elk.pm',
    'lib/Dist/Iller/Role/Config.pm',
    'lib/Dist/Iller/Types.pm',
    'lib/Dist/Zilla/MintingProfile/DistIller/Basic.pm',
    'lib/Dist/Zilla/Plugin/DistIller/MetaGeneratedBy.pm',
    'script/iller',
    't/00-compile.t',
    't/00-report-prereqs.dd',
    't/00-report-prereqs.t',
    't/02-builder.t',
    't/03-config.t',
    't/corpus/02-builder.yaml',
    't/corpus/03-config-config.yaml',
    't/corpus/03-config-iller.yaml',
    't/corpus/lib/Dist/Iller/Config/DistIllerTestConfig.pm'
);

eol_unix_ok($_, { trailing_whitespace => 1 }) foreach @files;
done_testing;
